﻿using System;
using System.IO; // pro třídu File
using System.Windows.Forms; // pro třídu Timer
using System.Diagnostics; // pro třídu PerformanceCounter
using System.Management; // tento jmenný prostor je nutné ručně přidat do referencí projektu

namespace Monitor
{
    static class Monitor
    {
        private static Timer timer = new Timer(); // interval čtení čítačů

        private static int interval = 1000; // výchozí interval (pokud není zadán na příkazové řádce)
        private static StreamWriter output = null; // výstupní soubor

        /*
         * Čítače lze v systému MS Windows zobrazit následovně:
         * Ovládací panely / Nástroje pro správu / Výkon
         * V kořenovém adresáři konzoly vybereme položku Sledování systému.
         * V pravé části okna klikneme na tlačítko Přidat a vybereme požadovaný čítač.
         * V aplikačním rozhraní je nutné používat anglické názvy, popřípadě tento program umožňuje zadat názvy kategorií, instancí a čítačů na příkazové řádce.
         */

        private static string cpuCategoryName = "Processor"; // jméno kategorie čítače procesoru
        private static string cpuInstanceName = "_Total"; // jméno instance čítače procesoru
        private static string cpuTimeCounterName = "% Processor Time"; // jméno čítače využitého času procesoru
        private static string memoryCategoryName = "Memory"; // jméno kategorie čítače paměti
        private static string memoryInstanceName = null; // jméno instance čítače paměti
        private static string memoryCommittedBytesCounterName = "% Committed Bytes In Use"; // jméno čítače procentuálního využití paměti
        private static string memoryAvailableBytesCounterName = "Available Bytes"; // jméno čítače počtu dostupných bytů paměti

        // objekty čítačů pro procesor a paměť
        private static PerformanceCounter cpuCounter = new PerformanceCounter();
        private static PerformanceCounter memoryCounter = new PerformanceCounter();

        // vstupní bod programu
        public static void Main(String[] param)
        {
            string outputFileName = "output.txt"; // výchozí jméno výstupního souboru (pokud není zadáno na příkazové řádce)

            if (param.Length == 1) // pokud je zadán 1 parametr ...
                if (param[0].CompareTo("-h") == 0 || param[0].CompareTo("--help") == 0) // ... a je to řětězec "-h" nebo "--help", program vypíše syntaxi použití programu
                {
                    System.Console.WriteLine("Use: Monitor.exe [output_file] [interval] [CPU category name] [CPU instance name] [CPU time counter name] [memory category name] [memory instance name] [memory committed bytes counter name] [memory available bytes counter name]");
                    return;
                }
            // pokud ja zadán alespoň 1 paramter, prvním paramtrem je jméno výstupního souboru
            if (param.Length > 0) outputFileName = param[0];
            // pokud jsou zadány alespoň 2 paramtery, druhým paramtrem je interval čtení čítačů v ms
            if (param.Length > 1) interval = int.Parse(param[1]);
            /*
             * Následující parametry slouží k ručnímu zadání jmen kategorií, instancí a čítačů.
             * Většinou není třeba zadávat, protože jména jsou většinou v angličtině, což je výchozí hodnotou.
             */
            if (param.Length > 2) cpuCategoryName = param[2];
            if (param.Length > 3)
            {
                cpuInstanceName = param[3];
                if (cpuInstanceName.CompareTo("") == 0)
                    cpuInstanceName = null;
            }
            if (param.Length > 4) cpuTimeCounterName = param[4];
            if (param.Length > 5) memoryCategoryName = param[5];
            if (param.Length > 6)
            {
                memoryInstanceName = param[6];
                if (memoryInstanceName.CompareTo("") == 0)
                    memoryInstanceName = null;
            }
            if (param.Length > 7) memoryCommittedBytesCounterName = param[7];
            if (param.Length > 8) memoryAvailableBytesCounterName = param[8];
            
            // nastavení vlastností časovače
            timer.Interval = interval; // interval časovače
            timer.Tick += new System.EventHandler(OnTick); // metoda, která se zavolá po každém uplynutí časovače

            output = new StreamWriter(outputFileName); // otevření souboru pro zápis

            /*
             * Zavoláme metodu OnTick, abychom hodnoty změřili jednou hned na začátku.
             * Jinak bychom museli čekat na první uplynutí časovače.
             */
            OnTick(null, null);

            timer.Start(); // časovač je spuštěn

            // program lze ukončit pouze zevnějšku (pod MS Windows klávesová zkratka Ctrl+C)
            System.Console.WriteLine("Press Ctrl+C to stop monitoring and exit.");

            // spustí smyčku zpráv programu
            Application.Run();
        }

        /*
         * Metoda sloužící ke čtení hodnot čítačů a jejich zápisu do výstupního souboru.
         * Volá se po každém uplynutí intervalu časovače.
         */
        private static void OnTick(object sender, System.EventArgs a)
        {
            // načtení hodnot čítačů a jejich převední na řetězce
            string cpu = GetProcessorData().ToString("F");
            string vmem = GetMemoryVData().ToString("F");
            string pmem = GetMemoryPData().ToString("F");

            // na systému MS Windows je nový řádek reprezentován dvojicí znaků - '\r' a '\n'
            output.Write('\r');
            output.Write('\n');
            output.Write(cpu); // zápis využití procesoru
            output.Write(' ');
            output.Write(pmem); // zápis využití fyzické paměťi
            //output.Write(vmem); // zápis využití virtuální paměťi
            output.Flush(); // vyrovnávací paměť výstupního souboru je zapsána na pevný disk (=uložení dat na disk)
        }

        // metoda sloužící pro načtení hodnot čítače pro procesor
        private static double GetProcessorData()
        {
            return GetCounterValue(cpuCounter, cpuCategoryName, cpuTimeCounterName, cpuInstanceName);
        }

        // metoda sloužící pro načtení hodnot čítače pro virtuální paměť
        private static double GetMemoryVData()
        {
            return GetCounterValue(memoryCounter, memoryCategoryName, memoryCommittedBytesCounterName, memoryInstanceName);
        }

        // metoda sloužící pro načtení hodnot čítače pro fyzickou paměť
        private static double GetMemoryPData()
        {
            // získáme hodnotu systémové vlastnosti celkové fyzické paměťi
            double totalphysicalmemory = Convert.ToDouble(QueryComputerSystem("totalphysicalmemory"));
            // získáme počet dostupných bytů pomocí čítače
            double d = GetCounterValue(memoryCounter, memoryCategoryName, memoryAvailableBytesCounterName, memoryInstanceName);
            // vypočítáme počet využitých bytů
            d = totalphysicalmemory - d;
            // vypočítáme využití fyzické paměti v intervalu <0;1>
            d /= totalphysicalmemory;
            // vynásobíme 100, abychom získali procentuální využití
            d *= 100;
            return d;
        }

        // metoda sloužící ke zjištění hodnoty čítače daných vlastností (jméno kategorie, čítače, instance)
        private static double GetCounterValue(PerformanceCounter pc, string categoryName, string counterName, string instanceName)
        {
            pc.CategoryName = categoryName;
            pc.CounterName = counterName;
            pc.InstanceName = instanceName;
            return pc.NextValue();
        }

        // metoda sloužící ke zjištění systémové vlastnosti (využito pro celkovou fyzickou paměť)
        private static string QueryComputerSystem(string query)
        {
            string value = null;
            // informace jsou uloženy v databázi, dotaz má formu SQL dotazu
            ManagementObjectSearcher manSearch = new ManagementObjectSearcher("SELECT * FROM Win32_ComputerSystem");
            // projdeme všechny vrácené výsledky, dokud není dosaženo požadované vlastnosti, jejíž hodnotu vrátíme
            foreach (ManagementObject manObject in manSearch.Get())
            {
                value = manObject[query].ToString();
            }
            return value;
        }
    }
}
